# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    MACAddressType,
    AttachFlexibleIPsResponse,
    DetachFlexibleIPsResponse,
    FlexibleIP,
    ListFlexibleIPsResponse,
    MACAddress,
    CreateFlexibleIPRequest,
    UpdateFlexibleIPRequest,
    AttachFlexibleIPRequest,
    DetachFlexibleIPRequest,
    GenerateMACAddrRequest,
    DuplicateMACAddrRequest,
    MoveMACAddrRequest,
)


def unmarshal_MACAddress(data: Any) -> MACAddress:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'MACAddress' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id")
    args["id"] = field

    field = data.get("mac_address")
    args["mac_address"] = field

    field = data.get("mac_type")
    args["mac_type"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return MACAddress(**args)


def unmarshal_FlexibleIP(data: Any) -> FlexibleIP:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'FlexibleIP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description")
    args["description"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("ip_address")
    args["ip_address"] = field

    field = data.get("mac_address")
    args["mac_address"] = unmarshal_MACAddress(field) if field is not None else None

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("reverse")
    args["reverse"] = field

    field = data.get("server_id")
    args["server_id"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("tags")
    args["tags"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return FlexibleIP(**args)


def unmarshal_AttachFlexibleIPsResponse(data: Any) -> AttachFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AttachFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips")
    args["flexible_ips"] = [unmarshal_FlexibleIP(v) for v in data["flexible_ips"]]

    field = data.get("total_count")
    args["total_count"] = field

    return AttachFlexibleIPsResponse(**args)


def unmarshal_DetachFlexibleIPsResponse(data: Any) -> DetachFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DetachFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips")
    args["flexible_ips"] = [unmarshal_FlexibleIP(v) for v in data["flexible_ips"]]

    field = data.get("total_count")
    args["total_count"] = field

    return DetachFlexibleIPsResponse(**args)


def unmarshal_ListFlexibleIPsResponse(data: Any) -> ListFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips")
    args["flexible_ips"] = [unmarshal_FlexibleIP(v) for v in data["flexible_ips"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListFlexibleIPsResponse(**args)


def marshal_AttachFlexibleIPRequest(
    request: AttachFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "fips_ids": request.fips_ids,
        "server_id": request.server_id,
    }


def marshal_CreateFlexibleIPRequest(
    request: CreateFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "is_ipv6": request.is_ipv6,
        "project_id": request.project_id or defaults.default_project_id,
        "reverse": request.reverse,
        "server_id": request.server_id,
        "tags": request.tags,
    }


def marshal_DetachFlexibleIPRequest(
    request: DetachFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "fips_ids": request.fips_ids,
    }


def marshal_DuplicateMACAddrRequest(
    request: DuplicateMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "duplicate_from_fip_id": request.duplicate_from_fip_id,
    }


def marshal_GenerateMACAddrRequest(
    request: GenerateMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "mac_type": MACAddressType(request.mac_type)
        if request.mac_type is not None
        else None,
    }


def marshal_MoveMACAddrRequest(
    request: MoveMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "dst_fip_id": request.dst_fip_id,
    }


def marshal_UpdateFlexibleIPRequest(
    request: UpdateFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "reverse": request.reverse,
        "tags": request.tags,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Zone,
)


class ListPrivateNetworksRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class ListPrivateNetworksResponse:
    private_networks: List[PrivateNetwork]

    total_count: int


@dataclass
class PrivateNetwork:
    """
    Private network.
    """

    id: str
    """
    Private Network ID.
    """

    name: str
    """
    Private Network name.
    """

    organization_id: str
    """
    Scaleway Organization the Private Network belongs to.
    """

    project_id: str
    """
    Scaleway Project the Private Network belongs to.
    """

    zone: Zone
    """
    Availability Zone in which the Private Network is available.
    """

    tags: List[str]
    """
    Tags of the Private Network.
    """

    created_at: Optional[datetime]
    """
    Date the Private Network was created.
    """

    updated_at: Optional[datetime]
    """
    Date the Private Network was last modified.
    """

    subnets: List[str]
    """
    Private Network subnets CIDR.
    """


@dataclass
class ListPrivateNetworksRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListPrivateNetworksRequestOrderBy]
    """
    Sort order of the returned Private Networks.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of Private Networks to return per page.
    """

    name: Optional[str]
    """
    Name to filter for. Only Private Networks with names containing this string will be returned.
    """

    tags: Optional[List[str]]
    """
    Tags to filter for. Only Private Networks with one or more matching tags will be returned.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for. Only Private Networks belonging to this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for. Only Private Networks belonging to this Project will be returned.
    """

    private_network_ids: Optional[List[str]]
    """
    Private Network IDs to filter for. Only Private Networks with one of these IDs will be returned.
    """

    include_regional: Optional[bool]
    """
    Defines whether to include regional Private Networks in the response.
    """


@dataclass
class CreatePrivateNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the Private Network.
    """

    project_id: Optional[str]
    """
    Scaleway Project in which to create the Private Network.
    """

    tags: Optional[List[str]]
    """
    Tags for the Private Network.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR.
    """


@dataclass
class GetPrivateNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """


@dataclass
class UpdatePrivateNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    name: Optional[str]
    """
    Name of the private network.
    """

    tags: Optional[List[str]]
    """
    Tags for the Private Network.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR (deprecated).
    :deprecated
    """


@dataclass
class DeletePrivateNetworkRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    ListProjectsResponse,
    Project,
    CreateProjectRequest,
    UpdateProjectRequest,
)


def unmarshal_Project(data: Any) -> Project:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Project' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description")
    args["description"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Project(**args)


def unmarshal_ListProjectsResponse(data: Any) -> ListProjectsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListProjectsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("projects")
    args["projects"] = [unmarshal_Project(v) for v in data["projects"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListProjectsResponse(**args)


def marshal_CreateProjectRequest(
    request: CreateProjectRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "name": request.name,
        "organization_id": request.organization_id or defaults.default_organization_id,
    }


def marshal_UpdateProjectRequest(
    request: UpdateProjectRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "name": request.name,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    PATRuleProtocol,
    DHCP,
    DHCPEntry,
    Gateway,
    GatewayNetwork,
    GatewayType,
    IP,
    ListDHCPEntriesResponse,
    ListDHCPsResponse,
    ListGatewayNetworksResponse,
    ListGatewayTypesResponse,
    ListGatewaysResponse,
    ListIPsResponse,
    ListPATRulesResponse,
    PATRule,
    SetDHCPEntriesRequestEntry,
    SetDHCPEntriesResponse,
    SetPATRulesRequestRule,
    SetPATRulesResponse,
    CreateGatewayRequest,
    UpdateGatewayRequest,
    CreateGatewayNetworkRequest,
    UpdateGatewayNetworkRequest,
    CreateDHCPRequest,
    UpdateDHCPRequest,
    CreateDHCPEntryRequest,
    UpdateDHCPEntryRequest,
    SetDHCPEntriesRequest,
    CreatePATRuleRequest,
    UpdatePATRuleRequest,
    SetPATRulesRequest,
    CreateIPRequest,
    UpdateIPRequest,
)


def unmarshal_DHCP(data: Any) -> DHCP:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DHCP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("address")
    args["address"] = field

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dns_local_name")
    args["dns_local_name"] = field

    field = data.get("dns_search")
    args["dns_search"] = field

    field = data.get("dns_servers_override")
    args["dns_servers_override"] = field

    field = data.get("enable_dynamic")
    args["enable_dynamic"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("pool_high")
    args["pool_high"] = field

    field = data.get("pool_low")
    args["pool_low"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("push_default_route")
    args["push_default_route"] = field

    field = data.get("push_dns_server")
    args["push_dns_server"] = field

    field = data.get("rebind_timer")
    args["rebind_timer"] = field

    field = data.get("renew_timer")
    args["renew_timer"] = field

    field = data.get("subnet")
    args["subnet"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("valid_lifetime")
    args["valid_lifetime"] = field

    field = data.get("zone")
    args["zone"] = field

    return DHCP(**args)


def unmarshal_GatewayNetwork(data: Any) -> GatewayNetwork:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'GatewayNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("address")
    args["address"] = field

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dhcp")
    args["dhcp"] = unmarshal_DHCP(field) if field is not None else None

    field = data.get("enable_dhcp")
    args["enable_dhcp"] = field

    field = data.get("enable_masquerade")
    args["enable_masquerade"] = field

    field = data.get("gateway_id")
    args["gateway_id"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("mac_address")
    args["mac_address"] = field

    field = data.get("private_network_id")
    args["private_network_id"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return GatewayNetwork(**args)


def unmarshal_GatewayType(data: Any) -> GatewayType:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'GatewayType' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("bandwidth")
    args["bandwidth"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("zone")
    args["zone"] = field

    return GatewayType(**args)


def unmarshal_IP(data: Any) -> IP:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'IP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("address")
    args["address"] = field

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("gateway_id")
    args["gateway_id"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("reverse")
    args["reverse"] = field

    field = data.get("tags")
    args["tags"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return IP(**args)


def unmarshal_DHCPEntry(data: Any) -> DHCPEntry:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DHCPEntry' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("gateway_network_id")
    args["gateway_network_id"] = field

    field = data.get("hostname")
    args["hostname"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("ip_address")
    args["ip_address"] = field

    field = data.get("mac_address")
    args["mac_address"] = field

    field = data.get("type_")
    args["type_"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return DHCPEntry(**args)


def unmarshal_Gateway(data: Any) -> Gateway:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Gateway' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("bastion_enabled")
    args["bastion_enabled"] = field

    field = data.get("bastion_port")
    args["bastion_port"] = field

    field = data.get("can_upgrade_to")
    args["can_upgrade_to"] = field

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("gateway_networks")
    args["gateway_networks"] = [
        unmarshal_GatewayNetwork(v) for v in data["gateway_networks"]
    ]

    field = data.get("id")
    args["id"] = field

    field = data.get("ip")
    args["ip"] = unmarshal_IP(field) if field is not None else None

    field = data.get("name")
    args["name"] = field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("smtp_enabled")
    args["smtp_enabled"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("tags")
    args["tags"] = field

    field = data.get("type_")
    args["type_"] = unmarshal_GatewayType(field) if field is not None else None

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("upstream_dns_servers")
    args["upstream_dns_servers"] = field

    field = data.get("version")
    args["version"] = field

    field = data.get("zone")
    args["zone"] = field

    return Gateway(**args)


def unmarshal_PATRule(data: Any) -> PATRule:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PATRule' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("gateway_id")
    args["gateway_id"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("private_ip")
    args["private_ip"] = field

    field = data.get("private_port")
    args["private_port"] = field

    field = data.get("protocol")
    args["protocol"] = field

    field = data.get("public_port")
    args["public_port"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone")
    args["zone"] = field

    return PATRule(**args)


def unmarshal_ListDHCPEntriesResponse(data: Any) -> ListDHCPEntriesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListDHCPEntriesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dhcp_entries")
    args["dhcp_entries"] = [unmarshal_DHCPEntry(v) for v in data["dhcp_entries"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListDHCPEntriesResponse(**args)


def unmarshal_ListDHCPsResponse(data: Any) -> ListDHCPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListDHCPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dhcps")
    args["dhcps"] = [unmarshal_DHCP(v) for v in data["dhcps"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListDHCPsResponse(**args)


def unmarshal_ListGatewayNetworksResponse(data: Any) -> ListGatewayNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListGatewayNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("gateway_networks")
    args["gateway_networks"] = [
        unmarshal_GatewayNetwork(v) for v in data["gateway_networks"]
    ]

    field = data.get("total_count")
    args["total_count"] = field

    return ListGatewayNetworksResponse(**args)


def unmarshal_ListGatewayTypesResponse(data: Any) -> ListGatewayTypesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListGatewayTypesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("types")
    args["types"] = [unmarshal_GatewayType(v) for v in data["types"]]

    return ListGatewayTypesResponse(**args)


def unmarshal_ListGatewaysResponse(data: Any) -> ListGatewaysResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListGatewaysResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("gateways")
    args["gateways"] = [unmarshal_Gateway(v) for v in data["gateways"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListGatewaysResponse(**args)


def unmarshal_ListIPsResponse(data: Any) -> ListIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("ips")
    args["ips"] = [unmarshal_IP(v) for v in data["ips"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListIPsResponse(**args)


def unmarshal_ListPATRulesResponse(data: Any) -> ListPATRulesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListPATRulesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pat_rules")
    args["pat_rules"] = [unmarshal_PATRule(v) for v in data["pat_rules"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListPATRulesResponse(**args)


def unmarshal_SetDHCPEntriesResponse(data: Any) -> SetDHCPEntriesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SetDHCPEntriesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dhcp_entries")
    args["dhcp_entries"] = [unmarshal_DHCPEntry(v) for v in data["dhcp_entries"]]

    return SetDHCPEntriesResponse(**args)


def unmarshal_SetPATRulesResponse(data: Any) -> SetPATRulesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SetPATRulesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pat_rules")
    args["pat_rules"] = [unmarshal_PATRule(v) for v in data["pat_rules"]]

    return SetPATRulesResponse(**args)


def marshal_CreateDHCPRequest(
    request: CreateDHCPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "address": request.address,
        "dns_local_name": request.dns_local_name,
        "dns_search": request.dns_search,
        "dns_servers_override": request.dns_servers_override,
        "enable_dynamic": request.enable_dynamic,
        "pool_high": request.pool_high,
        "pool_low": request.pool_low,
        "project_id": request.project_id or defaults.default_project_id,
        "push_default_route": request.push_default_route,
        "push_dns_server": request.push_dns_server,
        "rebind_timer": request.rebind_timer,
        "renew_timer": request.renew_timer,
        "subnet": request.subnet,
        "valid_lifetime": request.valid_lifetime,
    }


def marshal_SetDHCPEntriesRequestEntry(
    request: SetDHCPEntriesRequestEntry,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "ip_address": request.ip_address,
        "mac_address": request.mac_address,
    }


def marshal_SetPATRulesRequestRule(
    request: SetPATRulesRequestRule,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "private_ip": request.private_ip,
        "private_port": request.private_port,
        "protocol": PATRuleProtocol(request.protocol),
        "public_port": request.public_port,
    }


def marshal_CreateDHCPEntryRequest(
    request: CreateDHCPEntryRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "gateway_network_id": request.gateway_network_id,
        "ip_address": request.ip_address,
        "mac_address": request.mac_address,
    }


def marshal_CreateGatewayNetworkRequest(
    request: CreateGatewayNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        **resolve_one_of(
            [
                OneOfPossibility(
                    "dhcp_id", request.dhcp_id if request.dhcp_id is not None else None
                ),
                OneOfPossibility(
                    "dhcp",
                    marshal_CreateDHCPRequest(request.dhcp, defaults)
                    if request.dhcp is not None
                    else None,
                ),
                OneOfPossibility(
                    "address", request.address if request.address is not None else None
                ),
            ]
        ),
        "enable_dhcp": request.enable_dhcp,
        "enable_masquerade": request.enable_masquerade,
        "gateway_id": request.gateway_id,
        "private_network_id": request.private_network_id,
    }


def marshal_CreateGatewayRequest(
    request: CreateGatewayRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "bastion_port": request.bastion_port,
        "enable_bastion": request.enable_bastion,
        "enable_smtp": request.enable_smtp,
        "ip_id": request.ip_id,
        "name": request.name,
        "project_id": request.project_id or defaults.default_project_id,
        "tags": request.tags,
        "type": request.type_,
        "upstream_dns_servers": request.upstream_dns_servers,
    }


def marshal_CreateIPRequest(
    request: CreateIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "project_id": request.project_id or defaults.default_project_id,
        "tags": request.tags,
    }


def marshal_CreatePATRuleRequest(
    request: CreatePATRuleRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "gateway_id": request.gateway_id,
        "private_ip": request.private_ip,
        "private_port": request.private_port,
        "protocol": PATRuleProtocol(request.protocol),
        "public_port": request.public_port,
    }


def marshal_SetDHCPEntriesRequest(
    request: SetDHCPEntriesRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "dhcp_entries": [
            marshal_SetDHCPEntriesRequestEntry(v, defaults)
            for v in request.dhcp_entries
        ]
        if request.dhcp_entries is not None
        else None,
        "gateway_network_id": request.gateway_network_id,
    }


def marshal_SetPATRulesRequest(
    request: SetPATRulesRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "gateway_id": request.gateway_id,
        "pat_rules": [
            marshal_SetPATRulesRequestRule(v, defaults) for v in request.pat_rules
        ],
    }


def marshal_UpdateDHCPEntryRequest(
    request: UpdateDHCPEntryRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "ip_address": request.ip_address,
    }


def marshal_UpdateDHCPRequest(
    request: UpdateDHCPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "address": request.address,
        "dns_local_name": request.dns_local_name,
        "dns_search": request.dns_search,
        "dns_servers_override": request.dns_servers_override,
        "enable_dynamic": request.enable_dynamic,
        "pool_high": request.pool_high,
        "pool_low": request.pool_low,
        "push_default_route": request.push_default_route,
        "push_dns_server": request.push_dns_server,
        "rebind_timer": request.rebind_timer,
        "renew_timer": request.renew_timer,
        "subnet": request.subnet,
        "valid_lifetime": request.valid_lifetime,
    }


def marshal_UpdateGatewayNetworkRequest(
    request: UpdateGatewayNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        **resolve_one_of(
            [
                OneOfPossibility(
                    "dhcp_id", request.dhcp_id if request.dhcp_id is not None else None
                ),
                OneOfPossibility(
                    "address", request.address if request.address is not None else None
                ),
            ]
        ),
        "enable_dhcp": request.enable_dhcp,
        "enable_masquerade": request.enable_masquerade,
    }


def marshal_UpdateGatewayRequest(
    request: UpdateGatewayRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "bastion_port": request.bastion_port,
        "enable_bastion": request.enable_bastion,
        "enable_smtp": request.enable_smtp,
        "name": request.name,
        "tags": request.tags,
        "upstream_dns_servers": request.upstream_dns_servers,
    }


def marshal_UpdateIPRequest(
    request: UpdateIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "gateway_id": request.gateway_id,
        "reverse": request.reverse,
        "tags": request.tags,
    }


def marshal_UpdatePATRuleRequest(
    request: UpdatePATRuleRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "private_ip": request.private_ip,
        "private_port": request.private_port,
        "protocol": PATRuleProtocol(request.protocol),
        "public_port": request.public_port,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List, Optional

from scaleway_core.bridge import (
    Zone,
)
from .types import (
    Arch,
    BootType,
    Bootscript,
    Image,
    ImageState,
    PlacementGroup,
    PrivateNIC,
    SecurityGroup,
    SecurityGroupPolicy,
    SecurityGroupRule,
    SecurityGroupRuleAction,
    SecurityGroupRuleDirection,
    SecurityGroupRuleProtocol,
    SecurityGroupSummary,
    SecurityGroupTemplate,
    Server,
    ServerAction,
    ServerIp,
    ServerIpv6,
    ServerLocation,
    ServerMaintenance,
    ServerState,
    ServerSummary,
    Snapshot,
    SnapshotBaseVolume,
    SnapshotState,
    Volume,
    VolumeServerTemplate,
    VolumeSummary,
    VolumeVolumeType,
)


@dataclass
class _SetImageResponse:
    """
    Set image response.
    """

    image: Optional[Image]


@dataclass
class _SetSecurityGroupResponse:
    """
    Set security group response.
    """

    security_group: Optional[SecurityGroup]


@dataclass
class _SetSecurityGroupRuleResponse:
    """
    Set security group rule response.
    """

    rule: Optional[SecurityGroupRule]


@dataclass
class _SetServerResponse:
    """
    Set server response.
    """

    server: Optional[Server]


@dataclass
class _SetSnapshotResponse:
    """
    Set snapshot response.
    """

    snapshot: Optional[Snapshot]


@dataclass
class _CreateServerRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Instance name.
    """

    dynamic_ip_required: Optional[bool]
    """
    Define if a dynamic IP is required for the Instance.
    """

    commercial_type: str
    """
    Define the Instance commercial type (i.e. GP1-S).
    """

    image: str
    """
    Instance image ID or label.
    """

    volumes: Optional[Dict[str, VolumeServerTemplate]]
    """
    Volumes attached to the server.
    """

    enable_ipv6: bool
    """
    True if IPv6 is enabled on the server.
    """

    public_ip: Optional[str]
    """
    ID of the reserved IP to attach to the server.
    """

    boot_type: Optional[BootType]
    """
    Boot type to use.
    """

    bootscript: Optional[str]
    """
    Bootscript ID to use when `boot_type` is set to `bootscript`.
    :deprecated
    """

    organization: Optional[str]
    """
    Instance Organization ID.
    
    One-of ('project_identifier'): at most one of 'organization', 'project' could be set.
    :deprecated
    """

    project: Optional[str]
    """
    Instance Project ID.
    
    One-of ('project_identifier'): at most one of 'organization', 'project' could be set.
    """

    tags: Optional[List[str]]
    """
    Instance tags.
    """

    security_group: Optional[str]
    """
    Security group ID.
    """

    placement_group: Optional[str]
    """
    Placement group ID if Instance must be part of a placement group.
    """


@dataclass
class _SetServerRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    id: str
    """
    Instance unique ID.
    """

    name: str
    """
    Instance name.
    """

    organization: Optional[str]
    """
    Instance Organization ID.
    """

    project: Optional[str]
    """
    Instance Project ID.
    """

    allowed_actions: Optional[List[ServerAction]]
    """
    Provide a list of allowed actions on the server.
    """

    tags: Optional[List[str]]
    """
    Tags associated with the Instance.
    """

    commercial_type: str
    """
    Instance commercial type (eg. GP1-M).
    """

    creation_date: Optional[datetime]
    """
    Instance creation date.
    """

    dynamic_ip_required: bool
    """
    True if a dynamic IP is required.
    """

    enable_ipv6: bool
    """
    True if IPv6 is enabled.
    """

    hostname: str
    """
    Instance host name.
    """

    image: Optional[Image]
    """
    Provide information on the Instance image.
    """

    protected: bool
    """
    Instance protection option is activated.
    """

    private_ip: Optional[str]
    """
    Instance private IP address.
    """

    public_ip: Optional[ServerIp]
    """
    Information about the public IP.
    """

    modification_date: Optional[datetime]
    """
    Instance modification date.
    """

    state: ServerState
    """
    Instance state.
    """

    location: Optional[ServerLocation]
    """
    Instance location.
    """

    ipv6: Optional[ServerIpv6]
    """
    Instance IPv6 address.
    """

    bootscript: Optional[Bootscript]
    """
    Instance bootscript.
    :deprecated
    """

    boot_type: BootType
    """
    Instance boot type.
    """

    volumes: Optional[Dict[str, Volume]]
    """
    Instance volumes.
    """

    security_group: Optional[SecurityGroupSummary]
    """
    Instance security group.
    """

    maintenances: Optional[List[ServerMaintenance]]
    """
    Instance planned maintenances.
    """

    state_detail: str
    """
    Instance state_detail.
    """

    arch: Arch
    """
    Instance architecture (refers to the CPU architecture used for the Instance, e.g. x86_64, arm64).
    """

    placement_group: Optional[PlacementGroup]
    """
    Instance placement group.
    """

    private_nics: Optional[List[PrivateNIC]]
    """
    Instance private NICs.
    """


@dataclass
class _UpdateServerRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    server_id: str
    """
    UUID of the Instance.
    """

    name: Optional[str]
    """
    Name of the Instance.
    """

    boot_type: Optional[BootType]

    tags: Optional[List[str]]
    """
    Tags of the Instance.
    """

    volumes: Optional[Dict[str, VolumeServerTemplate]]

    bootscript: Optional[str]
    """
    :deprecated
    """

    dynamic_ip_required: Optional[bool]

    enable_ipv6: Optional[bool]

    protected: Optional[bool]

    security_group: Optional[SecurityGroupTemplate]

    placement_group: Optional[str]
    """
    Placement group ID if Instance must be part of a placement group.
    """

    private_nics: Optional[List[PrivateNIC]]
    """
    Instance private NICs.
    """


@dataclass
class _SetImageRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    id: str

    name: str

    arch: Arch

    creation_date: Optional[datetime]

    modification_date: Optional[datetime]

    default_bootscript: Optional[Bootscript]
    """
    :deprecated
    """

    extra_volumes: Optional[Dict[str, Volume]]

    from_server: str

    organization: Optional[str]

    public: bool

    root_volume: Optional[VolumeSummary]

    state: ImageState

    project: Optional[str]

    tags: Optional[List[str]]


@dataclass
class _SetSnapshotRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    snapshot_id: str

    id: str

    name: str

    organization: Optional[str]

    volume_type: VolumeVolumeType

    size: int

    state: SnapshotState

    base_volume: Optional[SnapshotBaseVolume]

    creation_date: Optional[datetime]

    modification_date: Optional[datetime]

    project: Optional[str]

    tags: Optional[List[str]]


@dataclass
class _SetSecurityGroupRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    id: str
    """
    ID of the security group (will be ignored).
    """

    name: str
    """
    Name of the security group.
    """

    tags: Optional[List[str]]
    """
    Tags of the security group.
    """

    creation_date: Optional[datetime]
    """
    Creation date of the security group (will be ignored).
    """

    modification_date: Optional[datetime]
    """
    Modification date of the security group (will be ignored).
    """

    description: str
    """
    Description of the security group.
    """

    enable_default_security: bool
    """
    True to block SMTP on IPv4 and IPv6. This feature is read only, please open a support ticket if you need to make it configurable.
    """

    inbound_default_policy: SecurityGroupPolicy
    """
    Default inbound policy.
    """

    outbound_default_policy: SecurityGroupPolicy
    """
    Default outbound policy.
    """

    organization: Optional[str]
    """
    Security groups Organization ID.
    """

    project: Optional[str]
    """
    Security group Project ID.
    """

    organization_default: Optional[bool]
    """
    Please use project_default instead.
    :deprecated
    """

    project_default: bool
    """
    True use this security group for future Instances created in this project.
    """

    servers: Optional[List[ServerSummary]]
    """
    Instances attached to this security group.
    """

    stateful: bool
    """
    True to set the security group as stateful.
    """


@dataclass
class _SetSecurityGroupRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    security_group_id: str

    security_group_rule_id: str

    id: str

    protocol: SecurityGroupRuleProtocol

    direction: SecurityGroupRuleDirection

    action: SecurityGroupRuleAction

    ip_range: str

    dest_port_from: Optional[int]

    dest_port_to: Optional[int]

    position: int

    editable: bool
